@@ -23,20 +23,20 @@ class Database:
         self.dbc.query_none(
             """
             create table if not exists "batch_log" (
-                id serial primary key,
-                priority int not null default 50,
-                ttl_seconds int, 
-                data text,
-                result text,
-                type varchar(255),
-                status varchar(255),
-                throttle_key varchar(63),
-                throttle_limit integer,
-                throttle_period integer, 
-                source text,
-                error text,
-                created_at timestamptz not null default now(),
-                updated_at timestamptz not null default now()
+                  id serial primary key
+                , priority int not null default 50
+                , ttl_seconds int
+                , data text
+                , result text
+                , type varchar(255)
+                , status varchar(255)
+                , throttle_key varchar(63)
+                , throttle_limit integer
+                , throttle_period integer
+                , source text
+                , error text
+                , created_at timestamptz not null default now()
+                , updated_at timestamptz not null default now()
             );
             comment on column batch_log.id is 'Unique internal id for this batch item';
             comment on column batch_log.priority is 'The batch item''s priority. Tasks with a higher number for priority will be picked first by executors. Tasks with the same number are equally important.';
@@ -128,44 +128,48 @@ $$ language plpgsql;
             """
                 insert into batch_log
                     (
-                    priority,
-                    ttl_seconds,
-                    data,
-                    result,
-                    type,
-                    status,
-                    throttle_key,
-                    throttle_limit,
-                    throttle_period,
-                    source
+                      priority
+                    , ttl_seconds
+                    , data
+                    , result
+                    , type
+                    , status
+                    , throttle_key
+                    , throttle_limit
+                    , throttle_period
+                    , source
                     )
                 values
                     (
-                    %(priority)s,
-                    %(ttl_seconds)s,
-                    %(data)s,
-                    %(result)s,
-                    %(type)s,
-                    %(status)s,
-                    %(throttle_key)s,
-                    %(throttle_limit)s,
-                    %(throttle_period)s,
-                    %(source)s
+                      %(priority)s
+                    , %(ttl_seconds)s
+                    , %(data)s
+                    , %(result)s
+                    , %(type)s
+                    , %(status)s
+                    , %(throttle_key)s
+                    , %(throttle_limit)s
+                    , %(throttle_period)s
+                    , %(source)s
                     )
-                on conflict(id)
-                do update
-                    set priority=%(priority)s,
-                    ttl_seconds=%(ttl_seconds)s,
-                    data=%(data)s,
-                    result=%(result)s,
-                    type=%(type)s,
-                    status=%(status)s,
-                    throttle_key=%(throttle_key)s,
-                    throttle_limit=%(throttle_limit)s,
-                    throttle_period=%(throttle_period)s,
-                    source=%(source)s,
-                    updated_at=now()
-                returning id
+                on
+                    conflict(id)
+                do
+                    update
+                set
+                      priority = %(priority)s
+                    , ttl_seconds = %(ttl_seconds)s
+                    , data = %(data)s
+                    , result = %(result)s
+                    , type = %(type)s
+                    , status = %(status)s
+                    , throttle_key = %(throttle_key)s
+                    , throttle_limit = %(throttle_limit)s
+                    , throttle_period = %(throttle_period)s
+                    , source = %(source)s
+                    , updated_at = now()
+                returning
+                    id
                 ;
                 """,
             batch_item,
@@ -176,22 +180,27 @@ $$ language plpgsql;
     def update_batch_item(self, batch_item):
         return self.dbc.query_one(
             """
-                update batch_log
-                    set priority=%(priority)s,
-                    ttl_seconds=%(ttl_seconds)s,
-                    data=%(data)s,
-                    result=%(result)s,
-                    type=%(type)s,
-                    status=%(status)s,
-                    throttle_key=%(throttle_key)s,
-                    throttle_limit=%(throttle_limit)s,
-                    throttle_period=%(throttle_period)s,
-                    source=%(source)s,
-                    error=%(error)s,
-                    updated_at=now()
-                where id = %(id)s
-                and updated_at = %(updated_at)s
-                returning updated_at
+                update
+                    batch_log
+                set
+                      priority = %(priority)s
+                    , ttl_seconds = %(ttl_seconds)s
+                    , data = %(data)s
+                    , result = %(result)s
+                    , type = %(type)s
+                    , status = %(status)s
+                    , throttle_key = %(throttle_key)s
+                    , throttle_limit = %(throttle_limit)s
+                    , throttle_period = %(throttle_period)s
+                    , source = %(source)s
+                    , error = %(error)s
+                    , updated_at = now()
+                where
+                    id = %(id)s
+                and
+                    updated_at = %(updated_at)s
+                returning
+                    updated_at
                 ;
                 """,
             batch_item,
@@ -201,14 +210,19 @@ $$ language plpgsql;
     def update_batch_hang_status(self):
         return self.dbc.query_none(
             """
-                update batch_log
-                    set
-                    status=%(to_status)s,
-                    updated_at=now()
-                where status = %(from_status)s
-                and ttl_seconds is not null
-                and ttl_seconds > 0
-                and extract('epoch' from ( now() - updated_at )) > ttl_seconds
+                update
+                    batch_log
+                set
+                      status=%(to_status)s
+                    , updated_at=now()
+                where
+                    status = %(from_status)s
+                and
+                    ttl_seconds is not null
+                and
+                    ttl_seconds > 0
+                and
+                    extract('epoch' from ( now() - updated_at )) > ttl_seconds
                 ;
                 """,
             {"from_status": (self.IN_PROGRESS,), "to_status": (self.HUNG,)},
@@ -218,11 +232,13 @@ $$ language plpgsql;
     def bump_batch_items(self, from_status=IN_PROGRESS, to_status=PENDING):
         return self.dbc.query_none(
             """
-                update batch_log
-                    set
-                    status=%(to_status)s,
-                    updated_at=now()
-                where status = %(from_status)s
+                update
+                    batch_log
+                set
+                     status = %(to_status)s
+                    , updated_at = now()
+                where
+                    status = %(from_status)s
                 ;
                 """,
             {"from_status": (from_status,), "to_status": (to_status,)},
@@ -234,13 +250,19 @@ $$ language plpgsql;
         if result is None:
             return self.dbc.query_one(
                 """
-                    update batch_log
-                        set status=%(status)s,
-                        error = %(error)s,
-                        updated_at=now()
-                    where id = %(id)s
-                    and updated_at = %(updated_at)s
-                    returning id, updated_at
+                    update
+                        batch_log
+                    set
+                          status=%(status)s
+                        , error = %(error)s
+                        , updated_at=now()
+                    where
+                        id = %(id)s
+                    and
+                        updated_at = %(updated_at)s
+                    returning
+                          id
+                        , updated_at
                     ;
                     """,
                 {"id": (id,), "status": (status,), "error": (error,), "updated_at": (updated_at,)},
@@ -248,14 +270,20 @@ $$ language plpgsql;
         else:
             return self.dbc.query_one(
                 """
-                    update batch_log
-                        set status=%(status)s,
-                        error = %(error)s,
-                        result = %(result)s,
-                        updated_at=now()
-                    where id = %(id)s
-                    and updated_at = %(updated_at)s
-                    returning id, updated_at
+                    update
+                        batch_log
+                    set
+                          status=%(status)s
+                        , error = %(error)s
+                        , result = %(result)s
+                        , updated_at=now()
+                    where
+                        id = %(id)s
+                    and
+                        updated_at = %(updated_at)s
+                    returning
+                          id
+                        , updated_at
                     ;
                     """,
                 {"id": (id,), "status": (status,), "error": (error,), "result": (result,), "updated_at": (updated_at,)},
@@ -265,10 +293,13 @@ $$ language plpgsql;
     def bump_batch_item_status(self, id, status):
         self.dbc.query_none(
             """
-                update batch_log
-                    set status=%(status)s,
-                    updated_at=now()
-                where id = %(id)s
+                update
+                    batch_log
+                set
+                      status = %(status)s
+                    , updated_at = now()
+                where
+                    id = %(id)s
                 ;
                 """,
             {"id": (id,), "status": (status,)},
@@ -278,19 +309,39 @@ $$ language plpgsql;
     def book_batch_item(self, from_status=PENDING, to_status=IN_PROGRESS):
         return self.dbc.query_one(
             """
-                update batch_log
-                    set status=%(to_status)s,
-                    updated_at=now()
-                where status = %(from_status)s
-                and id = (
-                    select id
-                    from batch_log
-                    where true
-                    and status = %(from_status)s
-                    order by priority desc, updated_at asc
-                    limit 1
-                )
-                returning id, priority, data, result, type, status, source, created_at, updated_at
+                update
+                    batch_log
+                set
+                      status=%(to_status)s
+                    , updated_at=now()
+                where
+                    status = %(from_status)s
+                and
+                    id = (
+                        select
+                            id
+                        from
+                            batch_log
+                        where
+                            true
+                        and
+                            status = %(from_status)s
+                        order by
+                              priority desc
+                            , updated_at asc
+                        limit
+                            1
+                    )
+                returning
+                      id
+                    , priority
+                    , data
+                    , result
+                    , type
+                    , status
+                    , source
+                    , created_at
+                    , updated_at
                 ;
                 """,
             {"from_status": (from_status,), "to_status": (to_status,)},
@@ -301,31 +352,70 @@ $$ language plpgsql;
     def book_throttled_batch_item(self, from_status=PENDING, to_status=IN_PROGRESS):
         return self.dbc.query_one(
             """
-                update batch_log
-                    set status=%(to_status)s,
-                    updated_at=now()
-                where status = %(from_status)s
-                and id = (
-                    select id
-                    from batch_log
-                    where true
-                    and status = %(from_status)s
-                    and gcra_throttle(throttle_key, throttle_limit, throttle_period, true)::int <=0
-                    order by priority desc, updated_at asc
-                    limit 1
-                )
-                returning id, priority, data, result, type, status, source, created_at, updated_at
+                update
+                    batch_log
+                set
+                     status=%(to_status)s
+                    , updated_at=now()
+                where
+                    status = %(from_status)s
+                and
+                    id = (
+                        select
+                            id
+                        from
+                            batch_log
+                        where
+                            true
+                        and
+                            status = %(from_status)s
+                        and
+                            gcra_throttle(throttle_key, throttle_limit, throttle_period, true)::int <=0
+                        order by
+                              priority desc
+                            , updated_at asc
+                        limit
+                            1
+                    )
+                returning
+                      id
+                    , priority
+                    , data
+                    , result
+                    , type
+                    , status
+                    , source
+                    , created_at
+                    , updated_at
                 ;
                 """,
             {"from_status": (from_status,), "to_status": (to_status,)},
         )
 
+    # Delete single item by id
+    def delete_by_id(self, id):
+        self.dbc.query_none(
+            """
+                delete from
+                    batch_log
+                set
+                      status = %(status)s
+                    , updated_at = now()
+                where
+                    id = %(id)s
+                ;
+                """,
+            {"id": (id,)},
+        )
+
     # Delete batch items with given status and update_at longer than given time
     def delete_batch_items_with_status(self, status):
         self.dbc.query_none(
             """
-                delete from batch_log
-                where status = %(status)s
+                delete from
+                    batch_log
+                where
+                    status = %(status)s
                 ;
                 """,
             {"status": (status,)},
@@ -335,7 +425,8 @@ $$ language plpgsql;
     def delete_all(self):
         return self.dbc.query_none(
             """
-                delete from batch_log
+                delete from
+                    batch_log
                 ;
                 """
         )
@@ -345,30 +436,42 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    id,
-                    priority,
-                    ttl_seconds,
-                    data,
-                    result,
-                    type,
-                    status,
-                    throttle_key,
-                    throttle_limit,
-                    throttle_period,
-                    source,
-                    created_at,
-                    updated_at
-                from batch_log
-                where true
-                and (%(id)s is null or id = any(%(id)s))
-                and (%(priority)s is null or priority = any(%(priority)s))
-                and (%(ttl_seconds)s is null or ttl_seconds = any(%(ttl_seconds)s))
-                and (%(type)s is null or type = any(%(type)s))
-                and (%(status)s is null or status = any(%(status)s))
-                and (%(throttle_key)s is null or throttle_key = any(%(throttle_key)s))
-                and (%(source)s is null or source = any(%(source)s))
-                order by priority desc, updated_at asc
-                limit %(limit)s
+                      id
+                    , priority
+                    , ttl_seconds
+                    , data
+                    , result
+                    , type
+                    , status
+                    , throttle_key
+                    , throttle_limit
+                    , throttle_period
+                    , source
+                    , created_at
+                    , updated_at
+                from
+                    batch_log
+                where
+                    true
+                and
+                    (%(id)s is null or id = any(%(id)s))
+                and
+                    (%(priority)s is null or priority = any(%(priority)s))
+                and
+                    (%(ttl_seconds)s is null or ttl_seconds = any(%(ttl_seconds)s))
+                and
+                    (%(type)s is null or type = any(%(type)s))
+                and
+                    (%(status)s is null or status = any(%(status)s))
+                and
+                    (%(throttle_key)s is null or throttle_key = any(%(throttle_key)s))
+                and
+                    (%(source)s is null or source = any(%(source)s))
+                order by
+                      priority desc
+                    , updated_at asc
+                limit
+                    %(limit)s
                 ;
                 """,
             {"id": (id,), "priority": (priority,), "type": (type,), "status": (status,), "throttle_key": (throttle_key,), "source": (source,), "limit": (limit,)},
@@ -379,23 +482,28 @@ $$ language plpgsql;
         return self.dbc.query_one(
             """
                 select
-                    id,
-                    priority,
-                    ttl_seconds,
-                    data,
-                    result,
-                    type,
-                    status,
-                    throttle_key,
-                    throttle_limit,
-                    throttle_period,
-                    source,
-                    created_at,
-                    updated_at
-                from batch_log
-                where id = %(id)s
-                order by priority desc, updated_at asc
-                limit 1
+                      id
+                    , priority
+                    , ttl_seconds
+                    , data
+                    , result
+                    , type
+                    , status
+                    , throttle_key
+                    , throttle_limit
+                    , throttle_period
+                    , source
+                    , created_at
+                    , updated_at
+                from
+                    batch_log
+                where
+                    id = %(id)s
+                order by
+                      priority desc
+                    , updated_at asc
+                limit
+                    1
                 ;
                 """,
             {"id": (id,)},
@@ -406,11 +514,14 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    count(*) as count,
-                    type as name,
-                    status as status
-                from batch_log
-                group by type, status
+                     count(*) as count
+                    , type as name
+                    , status as status
+                from
+                    batch_log
+                group by
+                      type
+                    , status
                 ;
                 """
         )
@@ -420,10 +531,12 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    count(*) as count,
-                    type as name
-                from batch_log
-                group by type
+                      count(*) as count
+                    , type as name
+                from
+                    batch_log
+                group by
+                    type
                 ;
                 """
         )
@@ -433,10 +546,12 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    count(*) as count,
-                    status as name
-                from batch_log
-                group by status
+                      count(*) as count
+                    , status as name
+                from
+                    batch_log
+                group by
+                    status
                 ;
                 """
         )
@@ -446,9 +561,10 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    count(*) as count,
-                    min(gcra_throttle(throttle_key, throttle_limit, throttle_period, false)) as wait_millis
-                from batch_log
+                      count(*) as count
+                    , min(gcra_throttle(throttle_key, throttle_limit, throttle_period, false)) as wait_millis
+                from
+                    batch_log
                 ;
                 """
         )
@@ -458,10 +574,12 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    id,
-                    gcra_throttle(throttle_key, throttle_limit, throttle_period, false) as wait_millis
-                from batch_log
-                order by wait_millis asc
+                      id
+                    , gcra_throttle(throttle_key, throttle_limit, throttle_period, false) as wait_millis
+                from
+                    batch_log
+                order by
+                    wait_millis asc
                 ;
                 """
         )
@@ -470,17 +588,17 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    type,
-                    status,
-                    count(*)
+                      type
+                    , status
+                    , count(*)
                 from
                     batch_log
                 group by
-                    type,
-                    status
+                      type
+                    , status
                 order by
-                    type,
-                    status
+                      type
+                    , status
                 ;
                 """
         )
@@ -490,32 +608,44 @@ $$ language plpgsql;
         return self.dbc.query_many(
             """
                 select
-                    id,
-                    priority,
-                    ttl_seconds, 
-                    data,
-                    result,
-                    type,
-                    status,
-                    throttle_key,
-                    throttle_limit,
-                    throttle_period,
-                    error,
-                    updated_at - created_at as runtime,
-                    extract(EPOCH from (updated_at - created_at)) as runtime_ts,
-                    updated_at,
-                    created_at
-                from batch_log
-                where true
-                and (%(id)s is null or id = any(%(id)s))
-                and (%(priority)s is null or priority = any(%(priority)s))
-                and (%(type)s is null or type = any(%(type)s))
-                and (%(status)s is null or status = any(%(status)s))
-                and (%(throttle_key)s is null or throttle_key = any(%(throttle_key)s))
-                and (%(error)s is null or error = any(%(error)s))
-                and (%(source)s is null or source = any(%(source)s))
-                order by updated_at desc, runtime asc, priority desc
-                limit %(limit)s
+                      id
+                    , priority
+                    , ttl_seconds
+                    , data
+                    , result
+                    , type
+                    , status
+                    , throttle_key
+                    , throttle_limit
+                    , throttle_period
+                    , error
+                    , updated_at - created_at as runtime
+                    , extract(EPOCH from (updated_at - created_at)) as runtime_ts
+                    , updated_at
+                    , created_at
+                from
+                    batch_log
+                where
+                    true
+                and
+                    (%(id)s is null or id = any(%(id)s))
+                and
+                    (%(priority)s is null or priority = any(%(priority)s))
+                and
+                    (%(type)s is null or type = any(%(type)s))
+                and
+                    (%(status)s is null or status = any(%(status)s))
+                and
+                    (%(throttle_key)s is null or throttle_key = any(%(throttle_key)s))
+                and
+                    (%(error)s is null or error = any(%(error)s))
+                and
+                    (%(source)s is null or source = any(%(source)s))
+                order by
+                      updated_at desc
+                    , runtime asc, priority desc
+                limit
+                    %(limit)s
                 ;
                 """,
             {"id": (id,), "priority": (priority,), "type": (type,), "status": (status,), "throttle_key": (throttle_key,), "source": (source,), "error": (error,), "limit": (limit,)},
@@ -524,7 +654,8 @@ $$ language plpgsql;
     def get_now(self):
         r = self.dbc.query_one(
             """
-                select now()
+                select
+                    now()
             ;"""
         )
         r = r.replace(tzinfo=pytz.UTC)
